#### Purpose:
## Backup live databases to RackSpace (cloud storage platform)
## The below is highly gem specific code that allows database snapshots to be uploaded to RackSpace's cloud platform

#### Value:
## Ruby driven backups, notifications of completion / failure and an audit trail.


#### Data:
## This will work if you have a Rackspace API key and account, as well as an environment yml full of database credentials.

require 'rubygems'
require 'yaml'
require 'pathname'
require 'openssl'
require 'net/scp'
require 'fog'
require 'active_support/time'

def exit_code(command)
  attempts = 0
  begin
  system(command) == true
  puts 'pass'
  rescue
    attempts += 1
    retry if attempts <= 3
    puts "#{command} has caused an error or has failed. Retrying. Attempt #{attempts}"
  end
end

begin
  environment = ARGV[0]
  current_directory = '/scripts/admin_scripts/'
  environment_data = YAML.load(File.read("#{current_directory}environments.yml"))["#{environment}"]
  database_name = environment_data['database_name']
  local_directory = environment_data['local_directory']
  backup_filename = environment_data['backup_filename']

  # Email array for Notifications
  email_notifications = ['foo@bar.com']
  # Date & Time Variables
  todays_time = Time.now().strftime("%Y%m%d")
  #Rackspace Variables
  segmentation_size = '5000MB' #Max 5KMB
  initialize_storage = Fog::Storage.new(
      :provider => 'rackspace',
      :rackspace_username => 'aFakeUserNameisHere',
      :rackspace_api_key => 'aFakeKeyisHere',
      :rackspace_region => :dfw
  )
  # Delete old Rackspace containers
  containername = Time.now().strftime('%Y%m')
  container2delete = (Time.now() - 13.months).strftime('%Y%m')
  dir2delete= initialize_storage.directories.each do |mycontainer|
    if mycontainer.key <= container2delete
      mycontainer.files.each do |file|
        file.destroy
      end
      mycontainer.destory if mycontainer.key <= container.delete
    end
  end
  # Create backup container
  unless directory = initialize_storage.directories.get(containername)
    directory = initialize_storage.directories.create(:key => containername)
    directory.public = false
    directory.save()
  end
  puts "Vacuum Database @ #{Time.now()}"
  exit_code("vacuumdb #{database_name} -U postgres")
  puts "Backup Database @ #{Time.now()}"
  exit_code("pg_dump -F c -f #{local_directory}#{environment}_#{todays_time}.backup -U postgres #{database_name}")
  puts "Tar'ing #{local_directory}#{backup_filename}#{environment}_#{todays_time}.rackup"
    if exit_code("nice -10 tar zcvf #{local_directory}#{environment}_#{todays_time}.tgz #{local_directory}#{environment}_#{todays_time}.backup") == 'true'
    puts "Passed initial TAR Requirements, proceeding... @ #{Time.now()}"
    end
  puts "Cleaning Rackups @ #{Time.now()}"
  exit_code("rm -f -r #{local_directory}*.backup")
  puts "Starting Split @ #{Time.now()}"
  exit_code("split -b #{segmentation_size} -d --additional-suffix=.rackup #{local_directory}#{environment}_#{todays_time}.tgz #{local_directory}#{environment}_#{todays_time}.part.")
  #Rackspace Upload data
  Pathname.new(local_directory).children.each do |child|
    if child.extname == '.rackup'
      filename = "#{child.dirname}/#{child.basename}"
      directory.files.create(:key  => "#{child.basename}", :body => File.open(filename, :public => false)) if !directory.files.head(filename)
    end
  end
  puts "Cleaning up @ #{Time.now()}"
  exit_code('rm -f -r /backups/*.rackup *.tgz')
  email_notifications.each {|fire| system("echo '' | mail -s 'Rackup of #{environment} has uploaded successfully.' #{fire}") }
rescue Exception => e
  puts "Sending Exception Notification @ #{Time.now()}, Exception => #{e.message} #{e.backtrace}"
  email_notifications.each {|fire| system("echo '' | mail -s 'Upload of #{environment} Rackup failed. #{e}' #{fire}") }
end
